#include "clustering.h"
#include <cstdio>
/**
 * \brief Cluster::Cluster()
 * Constructeur d'un cluster
 * \param id identifiant du cluster
 * \param Clustering pointeur vers le parent (pour avoir le table de RMSD
 */
Cluster::Cluster(int id,Clustering *parent)
{
    this->id = id;
    this->parent = parent;
    this->spread = 0;
}
/**
 * \brief Cluster::Cluster()
 * Constructeur par copie d'un cluster
 * \param var Cluster a copier
 */
Cluster::Cluster(const Cluster& var)
{
    this->id = var.id;
    this->parent = var.parent;
    for(unsigned int i = 0; i < var.m_proteine.size(); i++)
    {
        this->m_proteine.push_back(var.m_proteine.at(i));
    }
    this->spread = var.spread;
}
/**
 * \brief Cluster::addProtein()
 * Ajoute une proteine au cluster
 * \param prot Pointeur vers une proteine
 */
void Cluster::addProtein(Model *prot)
{
    //on ajoute l'adresse de la proteine dans le cluster
    m_proteine.push_back(prot);

}

/**
 * \brief Cluster::getid()
 * Permet de retrouver l'identifiant du cluster
 * \return Identifiant du cluster
 */
int Cluster::getid()
{
    return this->id;
}

/**
 * \brief Cluster::calculSpread()
 * Fonction qui va recalculer le Spread du cluster
 */
void Cluster::calculSpread()
{
    double var = 0;
    for(unsigned int k = 0; k < this->m_proteine.size(); k++)
    {
        for(unsigned int i = 0; i < this->m_proteine.size() && i < k; i++)
        {
            var += this->parent->getRMSD(this->m_proteine.at(i)->getID(),this->m_proteine.at(k)->getID());
        }
    }
    this->spread = var / ((double)((this->m_proteine.size() * this->m_proteine.size()) - this->m_proteine.size()) / 2);
    //std::cout << spread << std::endl;
}
/**
 * \brief Cluster::getSpread()
 * Permet de recuperer le Spread du cluster
 * \return Une valeur de Spread
 */
double Cluster::getSpread()
{
    return this->spread;
}

/**
 * \brief Cluster::getnbProtein()
 * Permet de recuperer le nombre de proteine du cluster
 * \return Le nombre de proteine du cluster
 */
unsigned int Cluster::getnbProtein()
{
    return m_proteine.size();
}

/**
 * \brief Cluster::proteinAt()
 * Permet de recuperer un pointeur vers une proteine du cluster
 * \param i Emplacement de la proteine
 * \return Un pointeur vers la proteine designier
 */
Model* Cluster::proteinAt(int i)
{
    return m_proteine.at(i);
}
