#include "distance.h"
#include <math.h>

/**
 * \brief calculRMSD()
 * Calcul le RMSD de deux proteine, ne prend que les Atome lourd
 * \param prot1 Reference vers une proteine
 * \param prot2 Reference vers une seconde proteine
 * \return Le RMSD entre ces deux proteine
 */
double calculRMSD(Model& prot1,Model& prot2)
{
    if(prot1.getnbAtomeofProteine() != prot2.getnbAtomeofProteine())
    {
        std::cout << "erreur 2 proteine n'on pas le meme nombre d'atome" << std::endl;
        return 0.0;
    }
    double result = 0;

    int nbAtomeLourd = 0;
    for(unsigned int i = 0; i < prot1.getnbAtomeofProteine(); i++)
    {
        if(prot1.Atomeat(i).type() != 'H')
        {
            nbAtomeLourd++;
            Point point1 = prot1.Atomeat(i).getCartesianCoord();
            Point point2 = prot2.Atomeat(i).getCartesianCoord();

            result += pow(point1.x - point2.x,2.0);
            result += pow(point1.y - point2.y,2.0);
            result += pow(point1.z - point2.z,2.0);
        }
    }

    //result *= (1.0/nbAtomeLourd); // tout les atome qui sont pas H (donc lourd)
    result = sqrt(result/nbAtomeLourd);
    //std::cout << "result : " << result << std::endl;
    return result;
}
