#include "entrypdb.h"
#include "reader.h"

/**
 * \brief Entrypdb::Entrypdb()
 * Constructeur d'une entrer PDB
 */
Entrypdb::Entrypdb()
{
}

/**
 * \brief Entrypdb::processLineFile()
 * Permet de traiter une ligne du fichier PDB
 * Cette fonction gere les Header et les model du fichier PDB
 * \param line Ligne du fichier PDB
 */
void Entrypdb::processLineFile(const char *line)
{

            Line_type linetype= typedeLigne(line);
            if (linetype == HEADER)
            {
                header.push_back(std::string(line));
            }
            else if ( linetype == DBREF)
            {
                header.push_back(std::string(line));
            }
            else if (linetype == SEQRES)
            {
                header.push_back(std::string(line));
            }
            else if ( linetype == MODEL)
            {
                int mnum=0;
                char buf[81];
                sscanf(line, "%s %d", buf, &mnum);
                model.push_back(Model(mnum));
            }
            else if (linetype== HETATM || linetype== ATOM || linetype== TER || linetype== ENDMDL)
            {
                if (model.empty())
                {
                    model.push_back(Model(0));
                }
                model.back().processLineModel(line);
            }
            else if (linetype== MASTER)
            {
            }

}

/**
 * \brief Entrypdb::Modelat()
 * Permet de recuperer une proteine a l'enplacement spécifier
 * \param i Emplacement de la proteine
 * \return Référence vers la proteine
 */
Model& Entrypdb::Modelat(int i)
{
    return this->model.at(i);
}
/**
 * \brief Entrypdb::getNbModel()
 * Permet de recuperer le nombre de proteine(MODEL) de l'entrer dans le fichier PDB
 * \return Nombre de Model(Proteine)
 */
unsigned int Entrypdb::getNbModel()
{
    return this->model.size();
}
/**
 * \brief Entrypdb::getheader()
 * Permet de recuperer le header de l'entrer PDB
 * \return Tout le header de l'entrer PDB
 */
std::string Entrypdb::getheader()
{
    std::string final;

    for (unsigned int i=0; i< header.size(); ++i)
    {
        final += header[i] + "\n";
    }
    return final;
}
/**
 * \brief Entrypdb::write()
 * Permet d'écrire l'entrer PDB dans un fichier PDB
 * \param out flux d'écriture
 */
void Entrypdb::write(std::ostream &out)
{
    for (unsigned int i=0; i< header.size(); ++i)
    {
        out << header[i] << std::endl;
    }
    for (unsigned int i=0; i< model.size(); ++i)
    {
        model[i].write(out);
    }
    out << "END   \n";
}

