#include "model.h"
#include "reader.h"
#include <assert.h>
#include <cstdlib>
Model::Model(int id)
{
    this->idinPDB = id;
    cree_chain_proteine = true;
}

/**
 * \brief Protein::setID()
 * Permet de definir un ID a une proteine l'id correspond a la ligne de la matrice du RMSD dans mon cas
 * \param id identifiant de la Proteine
 */
void Model::setID(int id)
{
    this->id = id;
}

/**
 * \brief Protein::getID()
 * Permet de l'ID de la proteine
 * \return l'identifiant de la proteine
 */
int Model::getID()
{
    return this->id;
}

/**
 * \brief Model::processLineModel()
 * Traite une ligne du fichier PDB,ajoute et gere les chaine de proteine de la class
 * \param ligne Ligne du fichier PDB
 */
void Model::processLineModel(const char *line) // UN MODELE = 1 PROTEINE , CONTIEN PLUSIEUR CHAINE DE PROTEINE
{
    Line_type typeLigne= typedeLigne(line);

    if (typeLigne == ATOM || typeLigne == HETATM)
    {

        int snum=-1;
        char name[5]={'\0'};
        char alt='\0';
        char resname[4]={'\0'};
        char chain;
        int resnum=-1;
        char insertion_residue_code;
        float x,y,z;
        float occupancy, tempFactor;
        char segID[5]={'\0'}, element[3]={'\0'}, charge[3]={'\0'};

        int numscan;
        if(typeLigne == ATOM)
        {
            numscan= sscanf(line, atom_line_iformat_,
            &snum, name, &alt, resname, &chain, &resnum, &insertion_residue_code,
            &x,&y,&z, &occupancy, &tempFactor, segID, element, charge);
        }
        else
        {
            numscan= sscanf(line, hetatm_line_iformat_,
            &snum, name, &alt, resname, &chain, &resnum, &insertion_residue_code,
            &x,&y,&z, &occupancy, &tempFactor, segID, element, charge);
        }
        assert(numscan >5);

        if(snum <= 0) // on ignore les ligne negative
        {
            m_Extrat.push_back(std::string(line));
            return;
        }
        if (m_ChainProtein.empty() || cree_chain_proteine)
        {
            m_ChainProtein.push_back(ChainProtein());
        }
        cree_chain_proteine = false;
        m_ChainProtein.back().process_line(line);
    }
    else if (typeLigne == TER)
    {
        cree_chain_proteine = true;
    }
    else
    {
        m_Extrat.push_back(std::string(line));
    }

}

/**
 * \brief Model::processCoord()
 * Traite une serie de coordonner comme un Atome et l'ajoute a la derniere chaine de proteine
 * Si il n'y a aucune chaine de proteine elle sera crée
 * \param x coordonner x de l'atome
 * \param y coordonner y de l'atome
 * \param z coordonner z de l'atome
 */
void Model::processCoord(double x, double y,double z)
{
    if (m_ChainProtein.empty())
    {
        m_ChainProtein.push_back(ChainProtein());
    }
    m_ChainProtein.back().process_coord(x, y, z);
}

/**
 * \brief Model::getnbAtomeofProteine()
 * Renvoie le nombre total d'atome de la proteine en incluant toute c'est chaine de proteine
 * \return Le nombre total d'atome de la proteine
 */
unsigned int Model::getnbAtomeofProteine()
{
    unsigned int result = 0;
    for(unsigned int i = 0 ; i < m_ChainProtein.size(); i++)
    {
        result += m_ChainProtein.at(i).getNbAtome();
    }
    return result;
}
/**
 * \brief Model::getnbAtomeLourdofProteine()
 * Renvoie le nombre total d'atome qui ne sont pas H de la proteine en incluant toute c'est chaine de proteine
 * \return Le nombre total d'atome Lourd de la proteine
 */
unsigned int Model::getnbAtomeLourdofProteine()
{
    unsigned int result = 0;
    for(unsigned int i = 0 ; i < m_ChainProtein.size(); i++)
    {
        result += m_ChainProtein.at(i).getNbAtomeLourd();
    }
    return result;
}
/**
 * \brief Model::Atomeat()
 * Renvoie l'atome  a la position demander,la focntion va chercher dans toute les chaine de proteine
 * l'atome demander.
 * \return L'atome a la position spécifier
 */
Atom& Model::Atomeat(unsigned int i)
{
    unsigned int depart = 0;
    unsigned int end = 0;
    for(unsigned int u = 0 ; u < m_ChainProtein.size(); u++)
    {
        depart = end;
        end = end + m_ChainProtein.at(u).getNbAtome();
        if( end > i)
        {
            return m_ChainProtein.at(u).atomeAt(i - depart);
        }
    }
    exit(-1);
}

/**
 * \brief Model::getNbChainProteine()
 * Retourne le nombre de chaine de proteine de la proteine
 * \return Le nombre de chaine de proteine
 */
unsigned int Model::getNbChainProteine()
{
    return this->m_ChainProtein.size();
}

/**
 * \brief Model::ChainAt()
 * Recupere le chaine de proteine a la position spécifier
 * \return Une référence vers la chain de proteine demander
 */
ChainProtein& Model::ChainAt(int i)
{
    return this->m_ChainProtein.at(i);
}

/**
 * \brief Model::write()
 * Permet d'ecrire au format PDB La proteine
 * \param out flux d'écriture
 */
void Model::write(std::ostream &out)
{
    char line[81];

    sprintf(line, "MODEL %8d         ", this->id);
    out << line << std::endl;
    for (unsigned int i=0; i< m_ChainProtein.size(); ++i)
    {
        m_ChainProtein[i].write(out);
    }
    for (unsigned int i=0; i< m_Extrat.size(); ++i)
    {
        out << m_Extrat[i] << std::endl;
    }
    out << "ENDMDL                       " << std::endl;
}
