#include "reader.h"
#include <iostream>
#include <cstdio>
#include <cstdlib>
/**
 * \brief reader::reader()
 * Constructeur d'un lecteur de pdb
 */
reader::reader()
{
}

/**
 * \brief typedeLigne()
 * Permet de recuperer le type de ligne lu dans le PDB
 * \param line Ligne lu dans le fichier pdb
 * \return Une enumeration du type de ligne rencontrer
 */
Line_type typedeLigne (const char* line)
{
    if (line[0]=='\0') return OTHER;
    std::string word(line,0,6);
    if (!word.empty() && (word[word.size() - 1] == '\r' || word[word.size() - 1] == '\n'))
        word.erase(word.size() - 1);
    if (word== "DBREF ") return DBREF;
    else if (word=="SEQRES") return SEQRES;
    else if (word== "ATOM  ") return ATOM;
    else if (word== "HETATM") return HETATM;
    else if (word== "MASTER") return MASTER;
    else if (word== "ENDMDL") return ENDMDL;
    else if (word== "END   " || word== "END  " || word== "END " || word== "END") return END;
    else if (   word == "HEADER" || word == "TITLE " || word == "COMPND" || word == "SOURCE"
          || word == "KEYWDS" || word == "EXPDTA" || word == "AUTHOR" || word == "REVDAT"
        || word == "JRNL  " || word == "REMARK" || word == "HELIX " || word == "SHEET "
        || word == "SITE  " || word == "CRYST1" || word == "ORIGX1" || word == "ORIGX2"
        || word == "ORIGX3" || word == "SCALE1" || word == "SCALE2" || word == "SCALE3"
        || word == "SEQADV" || word == "TURN  " || word == "FORMUL" || word == "HETNAM"
        || word == "SSBOND" || word == "MODRES" || word == "CONECT" || word == "HET   "
        || word == "CISPEP") return HEADER;
    else if (word== "TER   " || word == "TER") return TER;
    else if (word== "MODEL ") return MODEL;
    else
    {
      std::cout << "Instruction non reconnu dans le fichier : " << word << std::endl;
      return OTHER;
    }
}
/**
 * \brief reader::getNbEntryPDB()
 * Permet de recuperer le nombre d'entrer du fichier PDB
 * une entrer est un bloc qui va du header au premier END rencontrer
 * \return Le nombre d'entrer dans le fichier PDB
 */
unsigned int reader::getNbEntryPDB()
{
    return this->m_file.size();
}

/**
 * \brief reader::getentryPDBAt()
 * Permet de récuperer une référence vers une entrer dans le PDB
 * \param i Emplacement de l'entrer dans le PDB
 * \return Reférence vers une entrer PDB
 */
Entrypdb& reader::getentryPDBAt(unsigned int i)
{
    if(m_file.size() <= i)
    {
        std::cerr << "Depasement du tableau dans entryPDBAt" << std::endl;
        exit(-1);
    }
    return *(this->m_file.at(i));
}

/**
 * \brief reader::getNbProteine()
 * Permet de recuperer le nombre de proteine que le reader a en stock, fichierPDB + autre
 * \return Le nombre de proteine total
 */
unsigned int reader::getNbProteine()
{
   unsigned int nbProteine  = 0;
   for(unsigned int i = 0; i < m_file.size(); i++)
   {
           nbProteine +=  m_file.at(i)->getNbModel();
   }
   nbProteine += m_proteine.size();
   return nbProteine;
}
/**
 * \brief reader::getProteineAt()
 * Permet de récuperer une référence vers une Proteine
 * La fonction se charge elle meme de savoir ou est la proteine (Entrer PDB, model ou entrer de tableau)
 * \param i Emplacement de l'entrer
 * \return Reférence vers une Proteine
 */
Model& reader::getProteineAt(unsigned int i)
{
    unsigned int depart = 0;
    unsigned int end = 0;
    for(unsigned int u = 0 ; u < m_file.size(); u++)
    {
        depart = end;
        end = end + m_file.at(u)->getNbModel();
        if( end > i)
        {
            return m_file.at(u)->Modelat(i - depart);
        }
    }
    depart = end;
    end = end + m_proteine.size();
    if( end > i)
    {
        return m_proteine.at(i - depart);
    }
    std::cerr << "Depasement du tableau dans getProteineAt" << std::endl;
    exit(-1);
}

/**
 * \brief reader::readPDB()
 * Permet de lire un fichier PDB, il va crée automatiquement une entrer PDB a chaque END rencontrer
 * puis va envoiyer les ligne lu a l'entrer PDB en cour pour quelle crée les model,proteine et atome
 * \param in Flux d'entrer vers un fichier ou autre
 */
void reader::readPDB(std::istream &in)
{
        char line[600];
        bool createfile = true;
        while (in.getline (line, 600))
        {
            Line_type linetype= typedeLigne(line);
            if(createfile)
            {
                m_file.push_back(new Entrypdb());
                createfile = false;
            }
            if (linetype != END)
            {
                m_file.back()->processLineFile(line);
            }
            else
            {
               createfile = true;
            }
        }
}
/**
 * \brief reader::readdoubleTabinput()
 * Permet de lire un Tableau de coordonner et de l'ajouter en t'en que proteine
 * La fonction lit le tableau par 3 , x->case 1, y->case 2, z ->case 3, x->case4, ...
 * \param coord Tableau de coordonner
 * \param Nbentry Taille du tableau
 */
 void reader::readdoubleTabinput(double *coord, int Nbentry)
 {
     if(Nbentry % 3 != 0)
     {
         std::cerr << "Imposible d'ajouter le tableau car il manque des coordonner" << std::endl;
         return;
     }
    this->m_proteine.push_back(Model());
    for(int i = 0; i < Nbentry; i = i +3)
    {
        this->m_proteine.back().processCoord(coord[i], coord[i+1],coord[i+2]);
    }
 }

